import hashlib
import time
import requests
from httpClient.SdkException import *
from httpClient.RequestMethod import *
from httpClient.BaseHttpRequest import *


class SdkClient:

    def __init__(self, serverUrl, accessToken, accessSecret):
        self.__serverUrl = serverUrl
        self.__accessToken = accessToken
        self.__accessSecret = accessSecret

    def request(self, request):

        if not isinstance(request, BaseHttpRequest):
            raise ValueError('request must implement baseHttpRequest!')

        params = request.get_params()
        method = params.get_method()
        url = self.__serverUrl + request.get_path()

        timestamp = str(int(time.time() * 1000))
        headers = {'x-qys-open-signature': to_md5(self.__accessToken + self.__accessSecret + timestamp),
                   'x-qys-open-accesstoken': self.__accessToken,
                   'x-qys-open-timestamp': timestamp,
                   'version': "PYTHON-3.0.4"}

        while True:
            if method == RequestMethod.POST:
                if params.is_json():
                    data = params.get_json()
                    headers['Content-Type'] = 'application/json'
                    response = requests.post(url, data=data, headers=headers)
                else:
                    files = params.get_file()
                    data = params.get_params()
                    response = requests.post(url, data=data, headers=headers, files=files)
            elif method == RequestMethod.GET:
                data = params.get_params()
                response = requests.get(url, params=data, headers=headers)
            else:
                raise BaseSdkException(message="request method is invalid")

            if response.status_code == 200:
                return response.content
            else:
                raise HttpException(response.status_code, response.reason)


def to_md5(plain_text):
    md5 = hashlib.md5()
    md5.update(plain_text.encode('utf-8'))
    return md5.hexdigest()
